@@ -1,10 +1,9 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the zim.notebook module.'''
 
-from __future__ import with_statement
+
 
 import tests
 
@@ -13,7 +12,7 @@ import time
 
 from zim.fs import File, Dir
 from zim.newfs.mock import os_native_path
-from zim.config import ConfigManager, XDG_CONFIG_HOME, VirtualConfigBackend
+from zim.config import ConfigManager, XDG_CONFIG_HOME
 from zim.formats import ParseTree
 from zim.formats.wiki import Parser as WikiParser
 
@@ -36,7 +35,7 @@ class TestNotebookInfo(tests.TestCase):
 				# specifically ensure the "?" does not get url encoded
 		):
 			if os.name == 'nt':
-				if isinstance(location, basestring):
+				if isinstance(location, str):
 					location = location.replace('///', '///C:/')
 				uri = uri.replace('///', '///C:/')
 			info = NotebookInfo(location)
@@ -54,7 +53,7 @@ class TestNotebookInfoList(tests.TestCase):
 			file.remove()
 
 	def runTest(self):
-		root = Dir(self.create_tmp_dir(u'some_utf8_here_\u0421\u0430\u0439'))
+		root = Dir(self.create_tmp_dir('some_utf8_here_\u0421\u0430\u0439'))
 
 		# Start empty - see this is no issue
 		list = get_notebook_list()
@@ -185,10 +184,10 @@ class TestBuildNotebook(tests.TestCase):
 	# Test including automount and uniqueness !
 
 	def setUp(self):
-		self.tmpdir = Dir(self.get_tmp_name())
-		self.notebookdir = self.tmpdir.subdir('notebook')
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		self.notebookdir = folder.folder('notebook')
 
-		script = self.tmpdir.file('mount.py')
+		script = folder.file('mount.py')
 		script.write('''\
 import os
 import sys
@@ -225,12 +224,11 @@ mount=%s %s
 			(self.notebookdir.uri, None), # repeat to check uniqueness
 			(self.notebookdir.file('notebook.zim').uri, None),
 			(self.notebookdir.file('foo/bar.txt').uri, Path('foo:bar')),
-			#~ ('zim+' + tmpdir.uri + '?aaa:bbb:ccc', Path('aaa:bbb:ccc')),
 		):
-			#~ print ">>", uri
+			#~ print(">>", uri)
 			info = NotebookInfo(uri)
 			nb, p = build_notebook(info)
-			self.assertEqual(nb.dir, self.notebookdir)
+			self.assertEqual(nb.folder.path, self.notebookdir.path)
 			self.assertEqual(p, path)
 			if nbid is None:
 				nbid = id(nb)
@@ -245,8 +243,7 @@ mount=%s %s
 class TestNotebook(tests.TestCase):
 
 	def setUp(self):
-		path = self.get_tmp_name()
-		self.notebook = tests.new_notebook(fakedir=path)
+		self.notebook = self.setUpNotebook(content=tests.FULL_NOTEBOOK)
 
 	def testAPI(self):
 		'''Test various notebook methods'''
@@ -411,7 +408,7 @@ class TestNotebook(tests.TestCase):
 			'**bold** :AnotherNewPage\n')
 
 
-		#~ print '\n==== DB ===='
+		#~ print('\n==== DB ====')
 		#~ self.notebook.index.update()
 		#~ cursor = self.notebook.index.db.cursor()
 		#~ cursor.execute('select * from pages')
@@ -449,10 +446,11 @@ class TestNotebook(tests.TestCase):
 
 	def testResolveFile(self):
 		'''Test notebook.resolve_file()'''
+		from zim.fs import adapt_from_newfs, Dir
+		dir = Dir(self.notebook.folder.path) # XXX
+
 		path = Path('Foo:Bar')
-		dir = self.notebook.dir
 		self.notebook.config['Notebook']['document_root'] = './notebook_document_root'
-		self.notebook.do_properties_changed() # parse config
 		doc_root = self.notebook.document_root
 		self.assertEqual(doc_root, dir.subdir('notebook_document_root'))
 		for link, wanted, cleaned in (
@@ -506,7 +504,7 @@ class TestNotebook(tests.TestCase):
 
 	#~ def testResolveName(self):
 		#~ '''Test store.resolve_name().'''
-		#~ print '\n'+'='*10+'\nSTORE: %s' % self.store
+		#~ print('\n'+'='*10+'\nSTORE: %s' % self.store)
 #~
 		#~ # First make sure basic list function is working
 		#~ def list_pages(name):
@@ -527,9 +525,9 @@ class TestNotebook(tests.TestCase):
 			#~ ('FOO:Dus','Test:foo:bar','Test:foo:Dus'),
 			#~ # FIXME more ambigous test data
 		#~ ):
-			#~ print '-'*10+'\nLINK %s (%s)' % (link, namespace)
+			#~ print('-'*10+'\nLINK %s (%s)' % (link, namespace))
 			#~ r = self.store.resolve_name(link, namespace=namespace)
-			#~ print 'RESULT %s' % r
+			#~ print('RESULT %s' % r)
 			#~ self.assertEqual(r, name)
 
 
@@ -552,6 +550,31 @@ class TestNotebookCaseInsensitiveFileSystem(TestNotebook):
 		self.assertEqual(file2.read(), 'TEST 123')
 
 
+@tests.slowTest
+class TestEndOfLine(tests.TestCase):
+
+	def _test_eol(self, eol, literal_eol):
+		dir = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		config = NotebookConfig(dir.file('notebook.zim'))
+		config['Notebook']['endofline'] = eol
+		config.write()
+
+		notebook, x = build_notebook(dir)
+		page = notebook.get_page(Path('Test'))
+		page.parse('wiki', 'test 123\n456\n')
+		notebook.store_page(page)
+
+		with open(page.source_file.path, 'rb') as fh:
+			text = fh.read()
+			self.assertTrue(text.endswith(literal_eol))
+
+	def testUnix(self):
+		self._test_eol('unix', b'\n')
+
+	def testWindows(self):
+		self._test_eol('dos', b'\r\n')
+
+
 class TestUpdateLinksOnMovePage(tests.TestCase):
 
 	def getNotebookContent(self, notebook):
@@ -968,6 +991,17 @@ class TestPath(tests.TestCase):
 			self.assertEqual(path.namespace, namespace)
 			self.assertTrue(path.name in path.__repr__())
 
+		# test equality
+		path = self.generator('Foo:Bar')
+		self.assertTrue(path == Path('Foo:Bar'))
+		self.assertFalse(path == Path('Dus'))
+		self.assertTrue(path.ischild(Path('Foo')))
+		self.assertFalse(path.ischild(Path('Foo:Bar')))
+		self.assertTrue(path.match_namespace(Path('Foo')))
+		self.assertTrue(path.match_namespace(Path('Foo:Bar')))
+		self.assertFalse(path.match_namespace(Path('Foo:Bar:Baz')))
+
+
 	# TODO test operators on paths > < + - >= <= == !=
 
 
@@ -1147,28 +1181,10 @@ class TestPage(TestPath):
 
 class TestMovePageNewNotebook(tests.TestCase):
 
-	def setUp(self):
-		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_MOCK)
-		layout = FilesLayout(folder, endofline='unix')
-		index = Index(':memory:', layout)
-
-		### XXX - Big HACK here - Get better classes for this - XXX ###
-		dir = VirtualConfigBackend()
-		file = dir.file('notebook.zim')
-		file.dir = dir
-		file.dir.basename = 'Unnamed Notebook'
-		###
-		config = NotebookConfig(file)
-
-		dir = None
-		cache_dir = None
-		self.notebook = Notebook(dir, cache_dir, config, folder, layout, index)
-		index.check_and_update()
-
 	def runTest(self):
 		'''Try populating a notebook from scratch'''
 		# Based on bug lp:511481 - should reproduce bug with updating links to child pages
-		notebook = self.notebook
+		notebook = self.setUpNotebook()
 
 		for name, text in (
 			('page1', 'Foo bar\n'),
@@ -1177,7 +1193,7 @@ class TestMovePageNewNotebook(tests.TestCase):
 			('page3', 'Hmm\n'),
 		):
 			path = Path(name)
-			page = self.notebook.get_page(path)
+			page = notebook.get_page(path)
 			page.parse('wiki', text)
 			notebook.store_page(page)
 
@@ -1280,16 +1296,23 @@ class TestPageChangeFile(tests.TestCase):
 
 
 try:
-	import gio
+	from gi.repository import Gio
 except ImportError:
-	gio = None
+	Gio = None
 
 @tests.slowTest
-@tests.skipUnless(gio, 'Trashing not supported, \'gio\' is missing')
+@tests.skipUnless(Gio, 'Trashing not supported, \'gio\' is missing')
 class TestTrash(tests.TestCase):
 
 	def runTest(self):
-		notebook = tests.new_files_notebook(self.create_tmp_dir())
+		notebook = self.setUpNotebook(
+			mock=tests.MOCK_ALWAYS_REAL,
+			content={
+				'TrashMe': 'Test 123\n',
+				'TrashMe:sub1': 'Test 345\n',
+				'TrashMe:sub2': 'Test 345\n',
+			}
+		)
 		page = notebook.get_page(Path('TrashMe'))
 		self.assertTrue(page.exists())
 
@@ -1336,3 +1359,14 @@ class TestBackgroundSave(tests.TestCase):
 		text = page.dump('wiki')
 		self.assertEqual(text[-1], 'test 123\n')
 		self.assertEqual(signals['stored-page'], [(page,)]) # post handler happened as well
+
+
+class AttachmentsFolderIsinstance(tests.TestCase):
+
+	def runTest(self):
+		from zim.newfs import Folder
+		folder = self.setUpFolder()
+		layout = FilesLayout(folder)
+		afolder = layout.get_attachments_folder(Path('Test'))
+		self.assertIsInstance(afolder, Folder)
+		self.assertIsInstance(afolder, folder.__class__)  # Either LocalFolder or MockFolder
